package com.binary.weight.service;

import jakarta.annotation.Resource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import javax.sql.DataSource;
import java.util.List;
import java.util.Map;

@Service
public class SyncService {

    private JdbcTemplate sqlServerJdbcTemplate;

    private JdbcTemplate jdbcTemplate;

    @Resource
    private WeightService weightService;

    @Resource
    private SSEService sseService;

    Logger logger = LoggerFactory.getLogger(SyncService.class);

    public SyncService(@Qualifier("sqlServerDataSource") DataSource sqlServerDatasource, DataSource dataSource) {
        this.sqlServerJdbcTemplate = new JdbcTemplate(sqlServerDatasource);
        this.jdbcTemplate = new JdbcTemplate(dataSource);
    }

    public void sync() {
        long start = System.currentTimeMillis();
        String sql = "SELECT TOP 1000 * FROM tbl_weight_info WHERE id > ?";

        String id = jdbcTemplate.queryForObject("select max(id) from tbl_weight_info", String.class);
        List<Map<String, Object>> rows = sqlServerJdbcTemplate.queryForList(sql, id);

        // 动态生成插入语句
        if (!rows.isEmpty()) {
            StringBuilder insertSql = new StringBuilder("INSERT INTO tbl_weight_info (");

            // 获取列名并构建列部分的SQL
            String[] columnNames = rows.get(0).keySet().toArray(new String[0]);
            for (String column : columnNames) {
                insertSql.append("`").append(column).append("`").append(",");
            }
            insertSql.setLength(insertSql.length() - 1); // 去掉最后一个逗号
            insertSql.append(") VALUES (");

            // 构建值部分的SQL
            insertSql.append("?,".repeat(columnNames.length));
            insertSql.setLength(insertSql.length() - 1); // 去掉最后一个逗号
            insertSql.append(")");

            String finalInsertSql = insertSql.toString();

            // 准备批量插入的数据
            List<Object[]> batchArgs = rows.stream()
                    .map(row -> row.values().toArray())
                    .toList();

            // 执行批量插入
            jdbcTemplate.batchUpdate(finalInsertSql, batchArgs);
        }

        sseService.sendMessage(rows);
        weightService.removeCache();
        logger.info("同步数据：" + rows.size() + "条");
    }
}
