package com.binary.weight.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.annotation.Resource;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.zip.GZIPOutputStream;

@Service
public class WeightService {

    private byte[] cache;

    @Resource
    private JdbcTemplate jdbcTemplate;

    public byte[] getWeightData() {
        String sql = "SELECT net, carno, receiver, remark, goods, YEAR(gross_time) as `year`, MONTH(gross_time) as `month`, DAY(gross_time) as `day`, TIME(gross_time) as `time` FROM tbl_weight_info";

        if (cache == null) {
            ObjectMapper objectMapper = new ObjectMapper();

            try (
                    ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
                    GZIPOutputStream gzipOutputStream = new GZIPOutputStream(byteArrayOutputStream, 4096 * 8)
            ) {

                String json = objectMapper.writeValueAsString(jdbcTemplate.queryForList(sql));
                gzipOutputStream.write(json.getBytes(StandardCharsets.UTF_8));
                gzipOutputStream.finish();

                cache = byteArrayOutputStream.toByteArray();
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }

        return cache;
    }

    public void removeCache() {
        cache = null;
    }

}
